import java.util.Iterator;

public class CircularBoundedBuffer<A> implements BoundedQueue<A> {
  private A[] xs;
  private int front, back, size;

  CircularBoundedBuffer(int capacity) {
    xs    = (A[]) new Object[capacity];
    front = 0;
    back  = 0;
    size  = 0;
  }

  @Override
  public void enqueue(A x) {
    if (!full()) {
      xs[back] = x;
      back = inc(back);
      size++;
    }
  }

  @Override
  public A dequeue() {
    if (!empty()) {
      A x = xs[front];
      front = inc(front);
      size--;
      return x;
    }
    return null;
  }

  @Override
  public Boolean empty() {
    return size == 0;
  }

  protected Boolean full() {
    return size == xs.length;
  }

  @Override
  public int capacity() {
    return xs.length;
  }

  private int inc(int n) {
    return ++n % xs.length;
  }

  @Override
  public Iterator<A> iterator() {
    return new Iterator<A>() {
      private int n = front;

      @Override
      public boolean hasNext() {
        return n != back;
      }

      @Override
      public A next() {
        A x = xs[n];
        n = inc(n);
        return x;
      }
    };
  }
}
